<?php
namespace Estoque\Repositories;

class Custocmpprod extends \Doctrine\ORM\EntityRepository
{

    /**
     * Retorna o valor da funcao de custo produto.
     *
     * @param $cdfilsoli
     * @param $cdproduto
     * @param $dtentrega
     * @param $nrtipo
     * @return mixed
     * @throws \Exception
     */
    public function consultaFuncaoCustoProduto($cdfilsoli, $cdproduto, $dtentrega, $nrtipo){
//        /** @todo verificar por que a funcao somente pode ser executada com o formato YYYY/MM/DD*/
//        $dtentrega = \Geral\Util\DateUtil::getDataDeString($dtentrega);
//        $dtentrega = $dtentrega->format(\Geral\Util\DateUtil::FORMATO_AMERICANO);
        $sql = "
            SELECT CUSTO_PRODUTO (:P_CDFILSOLI, :P_CDPRODUTO, :P_NRTIPO, :P_DTENTREGA, SYSDATE) AS CUSTO
            FROM DUAL
        ";
        $paramsWhere = array(
            'P_CDFILSOLI'   => $cdfilsoli,
            'P_CDPRODUTO'   => $cdproduto,
            'P_NRTIPO'      => $nrtipo,
            'P_DTENTREGA'   => $dtentrega,
        );

        try {
            return $this->_em->getConnection()->fetchAll($sql, $paramsWhere);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function consultaFuncaoCustoProdutoLCEST(
        $filial, $cdProduto, $custo, $dtPosicao, $almox, $local, $lote, $subLote
    ){
        $sql = "
            SELECT CUSTO_PRODUTO_LCEST(:P_CDFILIAL, :P_CDPRODUTO, TO_NUMBER(NVL(:P_CUSTO, '3')), TO_DATE(:P_DTPOSICAO),
                                      SYSDATE, :P_CDALMOXARIFE, :P_CDLOCALESTOQ, :P_NRLOTEESTQ, :P_NRSUBLOTE) AS CUSTO
            FROM DUAL
        ";
        $paramsWhere = array(
            'P_CDFILIAL'     => $filial,
            'P_CDPRODUTO'    => $cdProduto,
            'P_CUSTO'        => $custo,
            'P_DTPOSICAO'    => $dtPosicao,
            'P_CDALMOXARIFE' => $almox,
            'P_CDLOCALESTOQ' => $local,
            'P_NRLOTEESTQ'   => $lote,
            'P_NRSUBLOTE'    => $subLote,
        );

        try {
            return $this->_em->getConnection()->fetchColumn($sql, $paramsWhere);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaUltimoCustoIndice1($dataReferencia)
    {
        $sql = <<<SQL
          SELECT VRINDICE
          FROM INDICE I,
            MOVINDICE M,
            PARAMGERAL P
          WHERE I.CDTPINDICE   = P.CDTPINDICE
            AND I.IDPERVALOR   = P.IDPERVALOR
            AND I.CDINDICE     = P.CDINDICEMF
            AND P.CDINDICEMF   IS NOT NULL
            AND I.CDTPINDICE   = M.CDTPINDICE
            AND I.IDPERVALOR   = M.IDPERVALOR
            AND I.CDINDICE     = M.CDINDICE
            AND M.DTREFIND     = :P_DTREFIND
SQL;
        $params = array(
            'P_DTREFIND' => $dataReferencia,
        );

        try {
            return $this->getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function retornaUltimoCustoIndice2($dataReferencia)
    {
        $sql = <<<SQL
          SELECT VRINDICE
          FROM INDICE I,
            MOVINDICE M,
            PARAMGERAL P
          WHERE I.CDTPINDICE   = P.CDTPINDICE2
            AND I.IDPERVALOR   = P.IDPERVALOR2
            AND I.CDINDICE     = P.CDINDICEMF2
            AND P.CDINDICEMF   IS NOT NULL
            AND I.CDTPINDICE   = M.CDTPINDICE
            AND I.IDPERVALOR   = M.IDPERVALOR
            AND I.CDINDICE     = M.CDINDICE
            AND M.DTREFIND     = :P_DTREFIND
SQL;
        $params = array(
            'P_DTREFIND' => $dataReferencia,
        );

        try {
            return $this->getEntityManager()->getConnection()->fetchArray($sql, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}